==> 1.0.0.tar.gz: ios/Classes/PlateformChannels/Shopping/ShoppingModulePlugin.swift <==
//
//  ShoppingModulePlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/2/21.
//

import Foundation
import Flutter
import FireworkVideo

class ShoppingModulePlugin: NSObject, FlutterPlugin, FireworkVideoShoppingDelegate, CartViewControllerProviding {
    var channel: FlutterMethodChannel! = nil
    private static var cartViewController: CartViewController?
    private var cartIconVisible = true
    private var itemCounts = 0
    private var productInfoViewConfigurator: ProductInfoViewConfigurable?

    
    private static func generateCallbackId() -> Int {
        struct CBIdIncreasingFactor {
            static var callbackId = 0
        }
        
        CBIdIncreasingFactor.callbackId += 1;
        return CBIdIncreasingFactor.callbackId
    }
    
    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: FWChannelName.FireworkShoppingModule.rawValue, binaryMessenger: registrar.messenger())
        let instance = ShoppingModulePlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case FWShoppingModuleMethodName.`init`.rawValue:
            initialize()
            result(nil)
        case FWShoppingModuleMethodName.setCartIconVisible.rawValue:
            if let visible = call.arguments as? Bool {
                setCartIconVisible(visible)
            }
            result(nil)
        case FWShoppingModuleMethodName.setCartItemCount.rawValue:
            if let count = call.arguments as? Int {
                setCartItemCount(count)
            }
            result(nil)
        case FWShoppingModuleMethodName.exitCartPage.rawValue:
            exitCartPage()
            result(nil)
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    func initialize() {
        DispatchQueue.once {
            FireworkVideoSDK.shopping.shoppingDelegate = self
            FireworkVideoSDK.shopping.cartViewControllerProvider = self
        }
    }
    
    func setCartIconVisible(_ visible: Bool) {
        cartIconVisible = visible
    }
    
    func setCartItemCount(_ itemCounts: Int) {
        self.itemCounts = itemCounts
        
        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }
        
        var shoppingCartIconConfiguration = rProductInfoViewConfigurator.shoppingCartIconConfiguration
        shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
        rProductInfoViewConfigurator.shoppingCartIconConfiguration = shoppingCartIconConfiguration
    }
    
    func exitCartPage() {
        ShoppingModulePlugin.cartViewController?.navigationController?.popViewController(animated: true)
        ShoppingModulePlugin.cartViewController = nil
    }
    
    // MARK: - FireworkVideoShoppingDelegate
    func fireworkShopping(_ fireworkShopping: FireworkVideoShopping, willDisplayProductInfo productInfoViewConfigurator: ProductInfoViewConfigurable, forVideo video: VideoDetails) {
        self.productInfoViewConfigurator = productInfoViewConfigurator
        
        channel.invokeMethod(ShoppingEventName.WillDisplayProduct.rawValue, arguments: ["videoId": video.videoID]) { [weak self] (result: Any?) -> Void in
            //The configuration of CartIcon'visible determined by global cartIconVisible
            if let cartIconVisible = self?.cartIconVisible {
                productInfoViewConfigurator.shoppingCartIconConfiguration.isHidden = !cartIconVisible
            }
            //The configuration of CartIcon'indicator'visible determined by global itemCounts
            if let itemCounts = self?.itemCounts {
                productInfoViewConfigurator.shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
            }
            
            let config = JsonHelper.fromDictionary(result as? [String: AnyObject], type: ProductInfoViewConfiguration.self)
            guard let rConfig = config else {
                return
            }
            
            ShoppingModulePlugin.hydrateProductViewConfig(rConfig, productInfoViewConfigurator)
        }
    }
    
    func fireworkShopping(_ fireworkShopping: FireworkVideoShopping, updateDetailsForProducts products: [ProductID], forVideo video: VideoDetails, _ productHydrator: ProductHydrating) {
        channel.invokeMethod(ShoppingEventName.UpdateProductDetails.rawValue, arguments: ["productIds": products]) { (result: Any?) -> Void in
                let products = RCTConvert.buildProducts(result as? [AnyObject])
                guard let rProducts = products else {
                    return
                }
            
                for product in rProducts {
                    productHydrator.hydrateProduct(product.productId, { build in
                        return ShoppingModulePlugin.hydrateProduct(product, build)
                    })
                }
            }
    }

    
    func fireworkShopping(_ fireworkShopping: FireworkVideoShopping, addProductVariantToCart item: SelectedProductVariant, fromVideo video: VideoDetails, _ addToCartCompletionHandler: @escaping AddToCartHandler) {
        channel.invokeMethod(ShoppingEventName.AddToCart.rawValue, arguments: ["productId": item.productID, "unitId": item.unitID]) { (result: Any?) -> Void in
            if let result = result as? [String : Any], let res = result["res"] as? String {
                let tips = result["tips"] as? String
                if res == "success" {
                    addToCartCompletionHandler(.feedbackOnly(.success(message: tips ?? "success")))
                } else {
                    addToCartCompletionHandler(.feedbackOnly(.failure(message: tips ?? "failure")))
                }
            }
        }
    }
    
    // MARK: - CartViewControllerProviding
    func cartViewController(for video: VideoDetails) -> FireworkVideo.CartViewController {
        let callbackId = ShoppingModulePlugin.generateCallbackId()
//
//        channel.invokeMethod(ShoppingEventName.ClickCartIcon.rawValue, arguments: nil) { (result: Any?) -> Void in
//            guard let entry = result as? [String: Any] else {
//                return
//            }
//
//            let noti = Notification(name: Notification.Name(rawValue: "showCustomCartView"), object: nil, userInfo: ["cbid": callbackId, "entryInfo": entry])
//            NotificationCenter.default.post(noti)
//        }
        
        ShoppingModulePlugin.cartViewController = CartViewController(callbackId: callbackId)
        return ShoppingModulePlugin.cartViewController!
    }
}

private extension ShoppingModulePlugin {
    static func hydrateProductViewConfig(_ config: ProductInfoViewConfiguration, _ productInfoViewConfigurator: ProductInfoViewConfigurable) {
        if let cartIcon = config.cartIcon {
            if let hidden = cartIcon.isHidden {
                var config = productInfoViewConfigurator.shoppingCartIconConfiguration
                config.isHidden = hidden
                productInfoViewConfigurator.shoppingCartIconConfiguration = config
            }
        }
        if let buttonConfiguration = config.addToCartButton {
            var config = productInfoViewConfigurator.productDetailsConfiguration
            if let backgroundColor = buttonConfiguration.backgroundColor  {
                config.addToCartButton.backgroundColor = backgroundColor.uicolor()
            }
            if let textColor = buttonConfiguration.textColor  {
                config.addToCartButton.textColor = textColor.uicolor()
            }
            if let fontSize = buttonConfiguration.fontSize  {
                config.addToCartButton.font = UIFont.systemFont(ofSize: CGFloat(fontSize))
            }
            productInfoViewConfigurator.productDetailsConfiguration = config
        }
    }
}

private extension ShoppingModulePlugin {
    static func hydrateProduct(_ product: Product, _ pbuild: ProductBuilder) -> ProductBuilder {
        if let name = product.name {
            pbuild.name(name)
        }
        if let description = product.description {
            pbuild.description(description)
        }
        if let variants = product.units {
            for variant in variants {
                pbuild.variant(variant.unitId) { pvBuild in
                    return ShoppingModulePlugin.buildVariant(variant, pvBuild)
                }
            }
        }
        return pbuild
    }
    
    static func buildVariant(_ variant: Product.ProductUnit, _ pvBuild: ProductVariantBuilder) -> ProductVariantBuilder {
        if let name = variant.name {
            pvBuild.name(name)
        }
        if let url = variant.url {
            pvBuild.url(url)
        }
        if let price = variant.price {
            if let amount = price.amount, let currencyCode = price.currencyCode {
                pvBuild.formattedPrice(Decimal(amount), currencyCode: currencyCode)
            }
        }
        let options = ShoppingModulePlugin.buildVariantOption(variant.options)
        if let options = options {
            pvBuild.options(options)
        }
        return pvBuild
    }
    
    static func buildVariantOption(_ options: [Product.VariantOption]?) -> [String: String]? {
        guard let options = options else {
            return nil
        }
        var result: [String: String] = [:]
        for option in options {
            guard let name = option.name, let value = option.value else {
                continue
            }
            result[name] = value
        }
        return result
    }
}

==> 1.3.0.tar.gz: ios/Classes/PlateformChannels/Shopping/ShoppingModulePlugin.swift <==
//
//  ShoppingModulePlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/2/21.
//

import Foundation
import Flutter
import FireworkVideo

class ShoppingModulePlugin: NSObject, FlutterPlugin, FireworkVideoShoppingDelegate, CartViewControllerProviding {
    var channel: FlutterMethodChannel! = nil
    static weak var cartViewController: FWCartViewController?
    private var customClickCartIconEnabled: Bool = false
    private var cartIconVisible = true
    private var itemCounts = 0
    private var productInfoViewConfigurator: ProductInfoViewConfigurable?

    
    private static func generateCallbackId() -> Int {
        struct CBIdIncreasingFactor {
            static var callbackId = 0
        }
        
        CBIdIncreasingFactor.callbackId += 1;
        return CBIdIncreasingFactor.callbackId
    }
    
    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: FWChannelName.FireworkShoppingModule.rawValue, binaryMessenger: registrar.messenger())
        let instance = ShoppingModulePlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case FWShoppingModuleMethodName.`init`.rawValue:
            initialize()
            result(nil)
        case FWShoppingModuleMethodName.setCartIconVisible.rawValue:
            if let visible = call.arguments as? Bool {
                setCartIconVisible(visible)
            }
            result(nil)
        case FWShoppingModuleMethodName.setCartItemCount.rawValue:
            if let count = call.arguments as? Int {
                setCartItemCount(count)
            }
            result(nil)
        case FWShoppingModuleMethodName.setCustomClickCartIconEnabled.rawValue:
            if let enabled = call.arguments as? Bool {
                customClickCartIconEnabled = enabled
            }
            result(nil)
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    func initialize() {
        DispatchQueue.once {
            FireworkVideoSDK.shopping.shoppingDelegate = self
            FireworkVideoSDK.shopping.cartViewControllerProvider = self
        }
    }
    
    func setCartIconVisible(_ visible: Bool) {
        cartIconVisible = visible
    }
    
    func setCartItemCount(_ itemCounts: Int) {
        self.itemCounts = itemCounts
        
        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }
        
        var shoppingCartIconConfiguration = rProductInfoViewConfigurator.shoppingCartIconConfiguration
        shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
        rProductInfoViewConfigurator.shoppingCartIconConfiguration = shoppingCartIconConfiguration
    }
    
    // MARK: - FireworkVideoShoppingDelegate
    func fireworkShopping(_ fireworkShopping: FireworkVideoShopping, willDisplayProductInfo productInfoViewConfigurator: ProductInfoViewConfigurable, forVideo video: VideoDetails) {
        self.productInfoViewConfigurator = productInfoViewConfigurator
        
        channel.invokeMethod(ShoppingEventName.WillDisplayProduct.rawValue, arguments: ["videoId": video.videoID]) { [weak self] (result: Any?) -> Void in
            //The configuration of CartIcon'visible determined by global cartIconVisible
            if let cartIconVisible = self?.cartIconVisible {
                productInfoViewConfigurator.shoppingCartIconConfiguration.isHidden = !cartIconVisible
            }
            //The configuration of CartIcon'indicator'visible determined by global itemCounts
            if let itemCounts = self?.itemCounts {
                productInfoViewConfigurator.shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
            }
            
            let config = JsonHelper.fromDictionary(result as? [String: AnyObject], type: ProductInfoViewConfiguration.self)
            guard let rConfig = config else {
                return
            }
            
            ShoppingModulePlugin.hydrateProductViewConfig(rConfig, productInfoViewConfigurator)
        }
    }
    
    func fireworkShopping(_ fireworkShopping: FireworkVideoShopping, updateDetailsForProducts products: [ProductID], forVideo video: VideoDetails, _ productHydrator: ProductHydrating) {
        channel.invokeMethod(ShoppingEventName.UpdateProductDetails.rawValue, arguments: ["productIds": products]) { (result: Any?) -> Void in
                let products = RCTConvert.buildProducts(result as? [AnyObject])
                guard let rProducts = products else {
                    return
                }
            
                for product in rProducts {
                    productHydrator.hydrateProduct(product.productId, { build in
                        return ShoppingModulePlugin.hydrateProduct(product, build)
                    })
                }
            }
    }

    
    func fireworkShopping(_ fireworkShopping: FireworkVideoShopping, addProductVariantToCart item: SelectedProductVariant, fromVideo video: VideoDetails, _ addToCartCompletionHandler: @escaping AddToCartHandler) {
        channel.invokeMethod(ShoppingEventName.AddToCart.rawValue, arguments: ["productId": item.productID, "unitId": item.unitID]) { (result: Any?) -> Void in
            if let result = result as? [String : Any], let res = result["res"] as? String {
                let tips = result["tips"] as? String
                if res == "success" {
                    addToCartCompletionHandler(.feedbackOnly(.success(message: tips ?? "success")))
                } else {
                    addToCartCompletionHandler(.feedbackOnly(.failure(message: tips ?? "failure")))
                }
            }
        }
    }
    
    // MARK: - CartViewControllerProviding
    func cartViewController(for video: VideoDetails) -> FireworkVideo.CartViewController {
        if customClickCartIconEnabled {
            ShoppingModulePlugin.cartViewController = nil
            channel.invokeMethod(
                ShoppingEventName.ClickCartIcon.rawValue,
                arguments: nil)
            return FWCartViewController(showCustomCartView: false)
        } else {
            let cartVC = FWCartViewController(showCustomCartView: true)
            ShoppingModulePlugin.cartViewController = cartVC
            return cartVC
        }
    }
}

private extension ShoppingModulePlugin {
    static func hydrateProductViewConfig(_ config: ProductInfoViewConfiguration, _ productInfoViewConfigurator: ProductInfoViewConfigurable) {
        if let buttonConfiguration = config.addToCartButton {
            var config = productInfoViewConfigurator.productDetailsConfiguration
            if let backgroundColor = buttonConfiguration.backgroundColor,
               backgroundColor.count > 0 {
                config.addToCartButton.backgroundColor = backgroundColor.uicolor()
            }
            if let textColor = buttonConfiguration.textColor,
               textColor.count > 0 {
                config.addToCartButton.textColor = textColor.uicolor()
            }
            if let fontSize = buttonConfiguration.fontSize  {
                config.addToCartButton.font = UIFont.systemFont(ofSize: fontSize)
            }
            productInfoViewConfigurator.productDetailsConfiguration = config
        }
    }
}

private extension ShoppingModulePlugin {
    static func hydrateProduct(_ product: Product, _ pbuild: ProductBuilder) -> ProductBuilder {
        if let name = product.name {
            pbuild.name(name)
        }
        if let description = product.description {
            pbuild.description(description)
        }
        if let variants = product.units {
            for variant in variants {
                pbuild.variant(variant.unitId) { pvBuild in
                    return ShoppingModulePlugin.buildVariant(variant, pvBuild)
                }
            }
        }
        return pbuild
    }
    
    static func buildVariant(_ variant: Product.ProductUnit, _ pvBuild: ProductVariantBuilder) -> ProductVariantBuilder {
        if let name = variant.name {
            pvBuild.name(name)
        }
        if let price = variant.price {
            if let amount = price.amount, let currencyCode = price.currencyCode {
                pvBuild.formattedPrice(Decimal(amount), currencyCode: currencyCode)
            }
        }
        if let url = variant.url {
            pvBuild.url(url)
        }
        if let imageUrl = variant.imageUrl {
            pvBuild.imageUrl(imageUrl)
        }
        let options = ShoppingModulePlugin.buildVariantOption(variant.options)
        if let options = options {
            pvBuild.options(options)
        }
        return pvBuild
    }
    
    static func buildVariantOption(_ options: [Product.VariantOption]?) -> [String: String]? {
        guard let options = options else {
            return nil
        }
        var result: [String: String] = [:]
        for option in options {
            guard let name = option.name, let value = option.value else {
                continue
            }
            result[name] = value
        }
        return result
    }
}
